#version 430 core

// Geometry shader: triangles on input, 18 vertices (= 6 triangles in our case) on output
layout (triangles) in;
layout (triangle_strip, max_vertices = 18) out;

// Input variables
in VertexData
{
	vec3 normal_ws;		// Normal in world space
	vec3 position_ws;	// Position in world space
	vec2 tex_coord;
} inData[3];

// Output variables
out VertexData
{
	vec3 normal_ws;		// Normal in world space
	vec3 position_ws;	// Position in world space
	vec2 tex_coord;
} outData;

// Data of all cameras
struct SingleCameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};
layout (std140, binding = 0) uniform CubeCameraData
{
	SingleCameraData cameras[6];
};

//-----------------------------------------------------------------------

void main()
{
	// Task 2: Use geometry shader to generate 6 triangles from the input triangle, one for each cubemap face.
	//		Each triangle must be transformed by the corresponding camera.
	//		Hint: For each triangle, pass the data of each vertex in inData to outData without any change. Compute
	//			gl_Position (use inData.position_ws and view and projection matrix of the proper camera), and set
	//			gl_Layer. Remember functions EmitVertex and EndPrimitive. Also remember, that inData is an array
	//			of three vertices of the input triangle. The data of all cameras are in array 'cameras'.
	for (int layer = 0; layer < 6; layer++)
	{
		for (int v = 0; v < 3; v++)
		{
			// Pass the data of the vertex
			outData.tex_coord = inData[v].tex_coord;
			outData.position_ws = inData[v].position_ws;
			outData.normal_ws = inData[v].normal_ws;

			// Set gl_Layer from 'layer'
			gl_Layer = layer;

			// Compute the gl_Position (using the proper camera)
			gl_Position = cameras[layer].projection * cameras[layer].view * vec4(inData[v].position_ws, 1.0);
			
			// Emit the vertex
			EmitVertex();
		}
		// Restart the primitive - create another triangle
		EndPrimitive();
	}
}
